package com.fzm.admin.schedule;

import cn.hutool.core.collection.CollectionUtil;
import cn.hutool.core.lang.TypeReference;
import cn.hutool.http.HttpUtil;
import cn.hutool.json.JSONUtil;
import com.fzm.common.entity.dto.CopyrightQueryRequest;
import com.fzm.common.entity.dto.CopyrightQueryResponse;
import com.fzm.common.enums.CopyrightApplyState;
import com.fzm.common.open.entity.dataobject.CopyrightApiDO;
import com.fzm.common.open.service.CopyrightApiService;
import com.fzm.common.properties.CopyrightProperties;
import com.fzm.common.utils.CopyrightSignUtil;
import com.fzm.common.utils.JsonUtil;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.redisson.Redisson;
import org.redisson.api.RLock;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.scheduling.annotation.Scheduled;
import org.springframework.stereotype.Component;
import org.springframework.transaction.annotation.Transactional;

import java.util.Date;
import java.util.List;
import java.util.TreeMap;
import java.util.concurrent.TimeUnit;

/**
 * @author tangtuo
 * @date 2022/1/10 15:17
 */
@Slf4j
@Component
@RequiredArgsConstructor(onConstructor = @__(@Autowired))
public class CopyrightApiTask {
    private final CopyrightApiService copyrightApiService;

    private final CopyrightProperties copyrightProperties;

    private final Redisson redisson;


    /**
     * 定时任务更新版权申请状态
     */
    @Scheduled(cron = "0 */10 * * * ?")
    @Transactional(rollbackFor = Exception.class)
    public void updateCopyrightState() throws InterruptedException {
        // 加锁,防止集群部署时,定时任务在多个节点上执行
        RLock lock = redisson.getLock("update:copyright:api:state");
        if (!lock.tryLock(30, TimeUnit.SECONDS)) {
            if (log.isDebugEnabled()) {
                log.debug("更新版权状态的定时任务已在其他节点运行");
            }
            return;
        }
        try {
            List<String> list = copyrightApiService.getSerialCodes();
            if (CollectionUtil.isEmpty(list)) {
                if (log.isDebugEnabled()) {
                    log.debug("当前无待更新状态的申请记录");
                }
                return;
            }
            for (String serialCode : list) {
                try {
                    log.info("更新状态: 当前流水号为: {}", serialCode);
                    CopyrightQueryRequest request = new CopyrightQueryRequest(serialCode, copyrightProperties.getAppId());
                    TreeMap<String, String> map = this.beanToMap(request, copyrightProperties.getAppSecret());
                    String sign = CopyrightSignUtil.generateSign(map);
                    request.setSign(sign);
                    String json = JSONUtil.toJsonStr(request);
                    log.info("查询审核结果, 当前流水号为:{} , 请求参数为 : {}", serialCode, json);
                    String response = HttpUtil.post(copyrightProperties.getQueryUrl(), json, 10000);
                    log.info("查询审核结果接口响应内容: {}", response);
                    if (StringUtils.isBlank(response)) {
                        log.error("版权局查询存证接口无响应, 流水号: {}", serialCode);
                        continue;
                    }
                    CopyrightQueryResponse copyrightResponse = JsonUtil.toObject(response, CopyrightQueryResponse.class);
                    if (copyrightResponse == null) {
                        log.error("版权局查询存证接口无响应, 流水号: {}", serialCode);
                        continue;
                    }
                    if (!copyrightResponse.isSuccess()) {
                        log.error("接口响应失败, 失败原因:{}, 流水号: {}", copyrightResponse.getMessage(), serialCode);
                        continue;
                    }
                    // 审核状态（0：未提交，1：待审核，2：同意，3：驳回，4：已提交，5：待终审，6：审核通过）
                    Integer audit_status = copyrightResponse.getAudit_status();
                    Integer registerState = getRegisterState(audit_status);
                    if (registerState == null) {
                        log.info("当前版权状态无更新, 流水号为: {}", serialCode);
                        continue;
                    }

                    CopyrightApiDO copyrightApiDO = copyrightApiService.getBySerialNum(serialCode);
                    if (registerState.equals(copyrightApiDO.getRegisterState())) {
                        log.info("当前版权状态无更新, 流水号为: {}", serialCode);
                        continue;
                    }

                    CopyrightApiDO apiDO = new CopyrightApiDO();
                    apiDO.setId(copyrightApiDO.getId());
                    apiDO.setRegisterState(registerState);
                    apiDO.setUpdateDate(new Date());
                    copyrightApiService.updateById(apiDO);
                } catch (Exception e) {
                    log.error("更新版权状态发送异常, 当前流水号为:{}", serialCode);
                    log.error(e.getMessage(), e);
                }
            }
        } finally {
            lock.unlock();
        }
    }

    private TreeMap<String, String> beanToMap(CopyrightQueryRequest request, String app_secret) {
        TreeMap<String, String> map = JSONUtil.toBean(JSONUtil.toJsonStr(request), new TypeReference<TreeMap<String, String>>() {
        }, true);
        map.put("app_secret", app_secret);
        return map;
    }

    private Integer getRegisterState(Integer audi_status) {
        // audi_status 审核状态（0：未提交，1：待审核，2：同意，3：驳回，4：已提交，5：待终审，6：审核通过）
        switch (audi_status) {
            case 1:
                return CopyrightApplyState.EXAMINE.getCode();
            case 5:
                return CopyrightApplyState.FINAL_JUDGMENT.getCode();
            case 3:
                return CopyrightApplyState.FAILED.getCode();
            case 6:
                return CopyrightApplyState.SUCCEEDED.getCode();
            default:
                return null;
        }
    }
}
