package com.fzm.portal.controller.open;

import com.fzm.common.constant.SystemConstant;
import com.fzm.common.entity.vo.CopyrightStateVo;
import com.fzm.common.enums.CopyrightApplyState;
import com.fzm.common.enums.ResultCode;
import com.fzm.common.exception.GlobalException;
import com.fzm.common.model.ResponseModel;
import com.fzm.common.open.entity.dataobject.CopyrightApiDO;
import com.fzm.common.open.entity.dataobject.OpenKeyDO;
import com.fzm.common.open.entity.openobject.BaseOO;
import com.fzm.common.open.entity.openobject.CopyrightAuthorOO;
import com.fzm.common.open.entity.openobject.CopyrightOO;
import com.fzm.common.open.entity.openobject.CopyrightOwnerOO;
import com.fzm.common.open.service.CopyrightApiService;
import com.fzm.common.open.service.OpenKeyService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.RequiredArgsConstructor;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;

import java.util.Date;
import java.util.List;

/**
 * @author wt
 * @date 2022/4/24
 */
@RestController
@Api(tags = "Api版权服务")
@RequestMapping("/api/copyright")
@RequiredArgsConstructor(onConstructor = @__(@Autowired))
public class CopyrightApiController {
    private final OpenKeyService openKeyService;
    private final CopyrightApiService copyrightApiService;

    @PostMapping("/apply")
    @ApiOperation("/申请版权")
    public ResponseModel<Integer> apply(@Validated @RequestBody BaseOO<CopyrightOO> baseOO) {
        String appKey = baseOO.getAppKey();
        OpenKeyDO openKeyDO = openKeyService.getByAppKey(appKey);
        if (openKeyDO == null) {
            throw GlobalException.newException(ResultCode.VALIDATE_FAILED);
        }

        String bizId = baseOO.getBizId();
        CopyrightApiDO apiDO = copyrightApiService.getByAppKeyAndBizId(appKey, bizId);
        if (apiDO != null) {
            return ResponseModel.success(apiDO.getId());
        }

        CopyrightOO copyrightOO = baseOO.getData();
        Integer publishState = copyrightOO.getPublishState();
        if (publishState == null || publishState != 0 && publishState != 1) {
            throw GlobalException.newException(ResultCode.VALIDATE_FAILED);
        }
        // 如果发表状态是已发表的话,需要传入首次发表日期和省市
        if (SystemConstant.BOOLEAN_DATA_TRUE.equals(publishState) &&
                (copyrightOO.getFirstPublishDate() == null ||
                        StringUtils.isAnyBlank(copyrightOO.getFirstPublishCity(), copyrightOO.getFirstPublishProvince()))) {
            throw GlobalException.newException(ResultCode.VALIDATE_FAILED);
        }

        List<CopyrightAuthorOO> authors = copyrightOO.getAuthors();
        for (CopyrightAuthorOO author : authors) {
            String signType = author.getSignType();
            if (!"本名".equals(signType) && !"别名".equals(signType) && !"匿名".equals(signType)) {
                throw GlobalException.newException(ResultCode.VALIDATE_FAILED);
            }
        }

        List<CopyrightOwnerOO> owners = copyrightOO.getOwners();
        for (CopyrightOwnerOO owner : owners) {
            Integer type = owner.getType();
            if (type != 0 && type != 1) {
                throw GlobalException.newException(ResultCode.VALIDATE_FAILED);
            }
            Integer isEffective = owner.getIsEffective();
            if (isEffective != 0 && isEffective != 1) {
                throw GlobalException.newException(ResultCode.VALIDATE_FAILED);
            }
            if (isEffective == 0) {
                Date endDate = owner.getEffectiveEndDate();
                if (endDate == null) {
                    throw GlobalException.newException(ResultCode.VALIDATE_FAILED);
                }
            }
            Integer idType = owner.getIdType();
            if (idType < 0 || idType > 7) {
                throw GlobalException.newException(ResultCode.VALIDATE_FAILED);
            }
            if (idType == 0) {
                String positivePhoto = owner.getPositivePhoto();
                String backPhoto = owner.getBackPhoto();
                String personalPhoto = owner.getPersonalPhoto();
                if (StringUtils.isAnyBlank(positivePhoto, backPhoto, personalPhoto)) {
                    throw GlobalException.newException(ResultCode.VALIDATE_FAILED);
                }
            } else {
                String certificatesPhoto = owner.getCertificatesPhoto();
                if (StringUtils.isBlank(certificatesPhoto)) {
                    throw GlobalException.newException(ResultCode.VALIDATE_FAILED);
                }
            }
        }

        Integer apply = copyrightApiService.apply(baseOO, openKeyDO);
        return ResponseModel.success(apply);
    }

    @GetMapping(value = "/state")
    @ApiOperation(value = "获取状态")
    public ResponseModel<CopyrightStateVo> getCopyrightState(@RequestParam(required = false) Integer id,
                                                             @RequestParam(required = false) String appKey,
                                                             @RequestParam(required = false) String bizId) {
        CopyrightApiDO apiDO = null;
        if (id != null) {
            apiDO = copyrightApiService.getById(id);
        }
        if (StringUtils.isNoneBlank(appKey, bizId)) {
            apiDO = copyrightApiService.getByAppKeyAndBizId(appKey, bizId);
        }
        if (apiDO == null) {
            throw GlobalException.newException(ResultCode.DATA_ERROR, "请核对后再重试");
        }
        CopyrightStateVo copyrightStateVo = new CopyrightStateVo(CopyrightApplyState.getByState(apiDO.getRegisterState()), apiDO.getRejectReason());
        return ResponseModel.success(copyrightStateVo);
    }
}
